"""
Program  : SDMinP
Version  : 1.0
Date     : 16.03.2005
Purpose  : Adjustment of the multiple type I error (FWER)
Programer: Markus Obreiter
Contact  : m.obreiter@dkfz-heidelberg.de
Copyright (C) 2005 Deutsches Krebsforschungszentrum, DKFZ (German Cancer
Research Center), Division of Clinical
Epidemiology


This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your option)
any later version. 
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

A copy of the GNU Public License is attached to the program documentation.
"""


import formatHelper

class CalculationGlobalPvalue:

    def __init__(self, appCfgObject):

        self.appConfig = appCfgObject

    def calcGlobalPvalueBecker (self, minRawPvaluesPermutVector, pvalRawHypoMin):
       
        # min p value per permutation
        globPval = -1
        count = 0
        B = len(minRawPvaluesPermutVector)

        for minRawPvaluePermut in minRawPvaluesPermutVector:
            if (minRawPvaluePermut <= pvalRawHypoMin):
                count += 1
            else:
                pass

        globPval = count / float(B) # float needed, otherwise / is interpreted as DIV 
        return globPval
        

    def calcGlobalP_simple(self, minQvector, markerList, logfile):
       
        # minQvector vector contains min p values for all permutations
        globPval = -1
        pvalRawOrgTemp = 1
        pvalRawOrgMin  = 1

        # get minimum unadjusted p value
        for m in markerList:
            pvalRawOrgTemp = m['pValRawOrg']
            if pvalRawOrgTemp < pvalRawOrgMin : pvalRawOrgMin = pvalRawOrgTemp
        
        globPval = self.calcGlobalPvalueBecker(minQvector, pvalRawOrgMin)  

        if logfile:
            qvectorFormattedSorted = []
            vectQSort = minQvector[:]            
            vectQSort.sort()

            hlpFormat = formatHelper.FormatHelper()            
                
            logfile.msgEoln("")
            logfile.msgEoln("Calculate global adjusted P value")
            logfile.msgEoln("=================================")
            logfile.msgEoln("")
            logfile.msgEoln("Qvector Formatted Sorted: " + str(hlpFormat.formatFloatList(vectQSort))) 
            logfile.msgEoln("Calculation Option      : SIMPLE")
            logfile.msgEoln("Minimum raw P value     : " + str(pvalRawOrgMin))
            logfile.msgEoln("globPval                : " + str(globPval))


        return globPval        

    def calcGlobalP_extendSecondSmallest(self, lstPValRawOrg, vectQSmallestRawPvalues, vectQSecondRawPvalues, logfile):

        # min p value per permutation
        globAdjPvalMessage = ""
        globAdjPval = -1
        count = 0
        B = len(vectQSmallestRawPvalues)
        posListSecond = []
        posListSecondInsufficient = []

        # get smallest and second smallest original raw p value
        lstPValRawOrg.sort()
        pvalRawHypoMin    = lstPValRawOrg[0]
        pvalRawHypoSecond = lstPValRawOrg[1]
        
        positionQ = -1
        for minRawPvaluePermut in vectQSmallestRawPvalues:
            positionQ = positionQ + 1
            if (minRawPvaluePermut < pvalRawHypoMin):
                count += 1
               
            elif (minRawPvaluePermut == pvalRawHypoMin):
                posListSecond.append(positionQ)
                if vectQSecondRawPvalues[positionQ] <= pvalRawHypoSecond :
                    count += 1
                if vectQSecondRawPvalues[positionQ] == pvalRawHypoSecond:
                    posListSecondInsufficient.append(positionQ)  # also the second raw p value comparison was insufficient

            else:
                pass

        if posListSecond:
            globAdjPvalMessage = "Used second raw p values for permutation positions (position starts at 0): " + str(posListSecond)
        if posListSecondInsufficient:
            globAdjPvalMessage = globAdjPvalMessage + "\nUsing second raw p value was not sufficient for permutation positions (position starts at 0): " + str(posListSecondInsufficient)

        globAdjPval = count / float(B) # float needed, otherwise / is interpreted as DIV

        if logfile:
            hlpFormat = formatHelper.FormatHelper() 
            logfile.msgEoln ("")
            logfile.msgEoln ("Calculate global adjusted P value")
            logfile.msgEoln ("=================================")
            logfile.msgEoln ("")
            logfile.msgEoln ("Calculation Option: EXTENDED")

            logfile.msgEoln ("")
            logfile.msg ("Qmin       : ")
            logfile.msgEoln (hlpFormat.formatFloatList(vectQSmallestRawPvalues))
            logfile.msg ("Qmin sorted: ")
            vectQSmallestRawPvalues.sort()
            logfile.msgEoln (hlpFormat.formatFloatList(vectQSmallestRawPvalues))
    
            logfile.msgEoln ("")
            logfile.msg ("Qsec       : ")
            logfile.msgEoln (hlpFormat.formatFloatList(vectQSecondRawPvalues))
            logfile.msg ("Qsec sorted: ")
            vectQSecondRawPvalues.sort()
            logfile.msgEoln (hlpFormat.formatFloatList(vectQSecondRawPvalues))
              
            logfile.msgEoln ("")
            lstPValRawOrg.sort()
            logfile.msg ("Raw p values (from observed test statistic) sorted: ")
            logfile.msgEoln(hlpFormat.formatFloatList(lstPValRawOrg))
    
            logfile.msgEoln("")
            logfile.msgEoln("Smallest raw P value from Original Statistic       : " + str(pvalRawHypoMin))
            logfile.msgEoln("Second smallest raw P value from Original Statistic: " + str(pvalRawHypoSecond))
        
            logfile.msgEoln("")
            if globAdjPvalMessage: logfile.msgEoln("Remark        : " + globAdjPvalMessage)            
            logfile.msgEoln("Global p-value: " + str(globAdjPval))

        
        return globAdjPval, globAdjPvalMessage
    
