"""
Program  : SDMinP
Version  : 1.0
Date     : 16.03.2005
Purpose  : Adjustment of the multiple type I error (FWER)
Programer: Markus Obreiter
Contact  : m.obreiter@dkfz-heidelberg.de
Copyright (C) 2005 Deutsches Krebsforschungszentrum, DKFZ (German Cancer
Research Center), Division of Clinical
Epidemiology

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your option)
any later version. 
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

A copy of the GNU Public License is attached to the program documentation.
"""

import ConfigParser, os, time, sys

class Config:

    def __init__(self, path, dataInputFilePath):
        
        dictParameter = {}

        ##############################################################################################       
        # Dynymic calculation tag                                                                    #
        ##############################################################################################

        # Calculation tag, consisting of year, month, day, hour, minutes, seconds: YYYYMMDDHHMiMiSS
        timeparameter       = time.localtime()
        currentDate         = "%i%i%i"  %(timeparameter[0],timeparameter[1],timeparameter[2])
        currentTime         = "%i%i%i"  %(timeparameter[3],timeparameter[4],timeparameter[5])
        self.calculationTag = currentDate + '_' + currentTime # the tag is used to label the result and log files

        ##############################################################################################       
        # Parameters from config file                                                                #
        ##############################################################################################             

        dictParameter['interactive_dialog']   = {'value': -1, 'valSet': [0, 1]}
        dictParameter['testCharacter']        = {'value': -1, 'valSet': ['RIGHT_SIDED','TWO_SIDED','LEFT_SIDED']}
        dictParameter['calcGlobalPvalOption'] = {'value': -1, 'valSet': ['SIMPLE', 'EXTENDED']}
        dictParameter['logRawPCalc']          = {'value': -1, 'valSet': [0, 1]}
        dictParameter['logSorting']           = {'value': -1, 'valSet': [0, 1]}
        dictParameter['logStepDown']          = {'value': -1, 'valSet': [0, 1]}
        dictParameter['logGlobalPvalCalc']    = {'value': -1, 'valSet': [0, 1]}
        dictParameter['resultRFormat']        = {'value': -1, 'valSet': [0, 1]}
        
        confFile = ConfigParser.ConfigParser()
        confFile.read(path)        

        self.interactiveDialog    = confFile.getint ('PROGRAM','INTERACTIVE_DIALOG')
        self.testCharacter        = confFile.get    ('CALCULATION','TEST_CHARACTER')               
        self.calcRawPFormula      = ""
        self.calcGlobalPvalOption = confFile.get    ('CALCULATION','GLOBAL_PVAL_OPTION')
        self.logRawPCalc          = confFile.getint ('LOG'   ,'RAW_P_CALCULATION')
        self.logSorting           = confFile.getint ('LOG'   ,'SORTING')
        self.logStepDown          = confFile.getint ('LOG'   ,'STEP_DOWN_PROCEDURE')
        self.logGlobalPvalCalc    = confFile.getint ('LOG'   ,'GLOBAL_PVAL_CALCULATION')
        self.resultRFormat        = confFile.getint ('RESULT','R_FORMATTED')

        dictParameter['interactive_dialog']['value']    = self.interactiveDialog
        dictParameter['testCharacter']['value']         = self.testCharacter 
        dictParameter['calcGlobalPvalOption']['value']  = self.calcGlobalPvalOption
        dictParameter['logRawPCalc']['value']           = self.logRawPCalc         
        dictParameter['logSorting']['value']            = self.logSorting         
        dictParameter['logStepDown']['value']           = self.logStepDown        
        dictParameter['logGlobalPvalCalc']['value']     = self.logGlobalPvalCalc
        dictParameter['resultRFormat']['value']         = self.resultRFormat

        for parameter in dictParameter.keys():
            if dictParameter[parameter]['value'] not in dictParameter[parameter]['valSet']:
                print 
                print "Invalid parameter given for %s" %str(parameter)
                print "  Valid parameters:"
                print "    %s" %str(dictParameter[parameter]['valSet'])
                sys.exit("Program terminated.")
            

        ##############################################################################################       
        # Parameters, check for presence in config file, otherwise set default values                #
        ##############################################################################################

        # split files: input file is divided in dynamic number of split files
        if confFile.has_section("SPLITFILES"):
            self.splitFilesMaxLines   = confFile.getint ('SPLITFILES','MAX_LINES')   # max number of lines allowed for a split file
            if self.splitFilesMaxLines < 1:
                self.useSplitFiles = 0            
            else:
                self.useSplitFiles = 1
            self.cleanSplitFileDirectory = confFile.getint ('SPLITFILES','DELETE_AFTER_USAGE')
        else:
            self.useSplitFiles        = 1      # split files are going to be created and used
            self.splitFilesMaxLines   = 100    # max number of lines allowed for a split file

          
        ##############################################################################################       
        # Parameters, set explicetely at this stage                                                  #
        ##############################################################################################

        # split file information
        self.splitFile_Directory     = '../temp' #ATTENTION: this directory will be completely cleaned before usage
        self.splitFile_PureName      = 'dataSplitFile'
        self.splitFile_Extension     = '.split'

        # input data file information
        self.dataInputFile_Path        = dataInputFilePath
        self.dataInputFile_Directory   = os.path.dirname(self.dataInputFile_Path)
        self.dataInputFile_PureFileName= os.path.splitext(os.path.basename(self.dataInputFile_Path))[0]

        # calculation id
        self.calculationID            = self.calculationTag + '_' + self.dataInputFile_PureFileName

        # log files
        self.logFileDirectory         = '../log'
        self.logFileCalcRawP_FileName = 'log_calcRawP.txt'
        self.logFileCalcRawP_Path     = self.logFileDirectory + os.sep + self.calculationID + '_' + self.logFileCalcRawP_FileName
        self.logFileSorting_FileName  = 'log_sorting.txt'
        self.logFileSorting_Path      = self.logFileDirectory + os.sep + self.calculationID + '_' + self.logFileSorting_FileName
        self.logFileStepDown_FileName = 'log_stepDown.txt'
        self.logFileStepDown_Path     = self.logFileDirectory + os.sep + self.calculationID + '_' + self.logFileStepDown_FileName
        self.logFileGlobal_FileName   = 'log_globalPval.txt'
        self.logFileGlobal_Path       = self.logFileDirectory + os.sep + self.calculationID + '_' + self.logFileGlobal_FileName

        # result files        
        self.resultFileDirectory  = '../result'
        self.resultFile_FileName  = '.result.txt'
        self.resultFile_Path      = self.resultFileDirectory + os.sep + self.calculationID + self.resultFile_FileName      
        self.rformatFile_FileName = '.result.R.txt'
        self.rformatFile_Path      = self.resultFileDirectory + os.sep + self.calculationID + self.rformatFile_FileName      
        

        
        
