"""
Program  : SDMinP
Version  : 1.0
Date     : 16.03.2005
Purpose  : Adjustment of the multiple type I error (FWER)
Programer: Markus Obreiter
Contact  : m.obreiter@dkfz-heidelberg.de
Copyright (C) 2005 Deutsches Krebsforschungszentrum, DKFZ (German Cancer
Research Center), Division of Clinical
Epidemiology

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your option)
any later version. 
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

A copy of the GNU Public License is attached to the program documentation.
"""

import formatHelper, time, sys
class MarkerListHandler:
    
    def __init__ (self, appCfgObject, lstMarkerList):

        self.appConfig      = appCfgObject
        self.lstMarkerList  = lstMarkerList
        self.testSideOption = self.appConfig.testCharacter
        if self.testSideOption not in ['RIGHT_SIDED', 'LEFT_SIDED', 'TWO_SIDED']:            
            sys.exit("No valid option entered for test option in module markerListHandler.py")       

    def addMarker(self, marker):
        self.lstMarkerList.append(marker)

    def setMarkerList(self, markList):
        self.lstMarkerList = markerList        

    def getMarkerList(self):
        return self.lstMarkerList[:]
               

    def sortMarkerList(self, logfile):

        markerListSortedByOrgPvalueDesc = []            #container for the result markers, sorted by orig. pvalue descending
        for i in range(len(self.lstMarkerList)):        # initialize the positions
            markerListSortedByOrgPvalueDesc.append(-1)             

        # prepare a list of original raw p values
        listRawPOrg = []
        for marker in self.lstMarkerList:
            listRawPOrg.append(marker['pValRawOrg'])        

        listRawPOrg.sort()
        listRawPOrg.reverse()
        

        # assign the rank per marker
        for marker in self.lstMarkerList:           
            indx = listRawPOrg.index(marker['pValRawOrg'])  # returns lowest index for pval

            # if the indx position is occupied, go to next one                
            while markerListSortedByOrgPvalueDesc[indx] != -1:  
                indx = indx + 1

            # assign marker to the right order position
            markerListSortedByOrgPvalueDesc[indx] = marker

        i = 0
        if logfile:
            logfile.msgEoln('Tests, ordered by raw p value descending')
            logfile.msgEoln('----------------------------------------')
            logfile.msgEoln('ID\tRAW-P')
            for marker in markerListSortedByOrgPvalueDesc:
                s = str(marker['id']) + '\t' + str(marker['pValRawOrg'])
                logfile.msgEoln(s)            
            
        return markerListSortedByOrgPvalueDesc

    def getLstRawPValOrg(self, logfile):

        # create list of raw p values from original statistic
        vectorPvalRawOrg = []
        for m in self.lstMarkerList:           
            vectorPvalRawOrg.append (m['pValRawOrg'])

        if logfile:
            hlpFormat = formatHelper.FormatHelper()
            logfile.msgEoln ("")
            logfile.msgEoln ("List of original raw p values")
            logfile.msgEoln ("=============================")
            logfile.msgEoln (hlpFormat.formatFloatList(vectorPvalRawOrg))
            
        return vectorPvalRawOrg
          

    def getSmallestSecondSmallesRawPofPermutations (self, logfile):

        vectRawPValuesPermut = []
        position = -1
        permutation = {}
        permutationVector = []
        AMOUNT_PERMUTATIONS = self.lstMarkerList[0]['amount_perm']

        # create list of raw p values from permutations, by marker
        for m in self.lstMarkerList:
            vectRawPValuesPermut.append(m['lstPvalRawPrm'][:])

        # ingoing "vectRawPValuesPermut" keeps raw p values for each permutation, grouped by marker
        # this has to be changed, so that raw p values are grouped by permutation

        # create vector of pvalues per permutation i
        for i in range(AMOUNT_PERMUTATIONS):
            permutation [i] = []
            for rawPvalueVector in vectRawPValuesPermut:
                permutation[i].append(rawPvalueVector[i])
            permutationVector.append(permutation[i])

        # get smallest (for debug reasons) and second smallest raw p value of i-th permutation over all markers
        minRawPvector = []
        secMinRawPvector = []
        for i in range(AMOUNT_PERMUTATIONS):
            permutationTmp = permutation[i][:]
            permutationTmp.sort()
            minRawPvector.append(permutationTmp[0])      # final result of that vector has to be the same as last Q vector
            secMinRawPvector.append(permutationTmp[1])   # this vector keeps the second smalles raw p values for permutations

        if logfile:
            hlpFormat = formatHelper.FormatHelper()
            
            logfile.msgEoln ("")
            logfile.msgEoln ("Get smallest and second smallest raw p values from permutations")
            logfile.msgEoln ("===============================================================")
            for i in range(AMOUNT_PERMUTATIONS):

                logfile.msgEoln ("")
                logfile.msgEoln ("Raw P values (no order) of permutation replicate " + str (i+1) + ":")
                logfile.msgEoln(hlpFormat.formatFloatList(permutation[i]))
    
            logfile.msgEoln ("")
            logfile.msgEoln ("minRawPvectorFormatted (has to be the same as last 'Qvector New Formatted'):")
            logfile.msgEoln(hlpFormat.formatFloatList(minRawPvector))        
    
            logfile.msgEoln ("")
            logfile.msgEoln ("secMinRawPvectorFormatted:")
            logfile.msgEoln(hlpFormat.formatFloatList(secMinRawPvector))


        return minRawPvector, secMinRawPvector
        
    def debugLogMarkerList(self):
        fp = open ("debug.txt","a")
        fp.write ("----------------------\n")
        fp.write (time.asctime() + "\n")
        keyList = self.lstMarkerList[0].keys()
        # headline
        for key in keysList:
            fp.write(key)
        fp.write("\n")
        for marker in self.lstMarkerList:
            for key in keyList:
                msg = marker[key] + "\t"
            msg = msg + "\n"
            fp.write(msg)
        fp.close()
        
