"""
Program  : SDMinP
Version  : 1.0
Date     : 16.03.2005
Purpose  : Adjustment of the multiple type I error (FWER)
Programer: Markus Obreiter
Contact  : m.obreiter@dkfz-heidelberg.de
Copyright (C) 2005 Deutsches Krebsforschungszentrum, DKFZ (German Cancer
Research Center), Division of Clinical
Epidemiology

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your option)
any later version. 
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

A copy of the GNU Public License is attached to the program documentation.
"""

import logfile, sys

class Resultfile:
    
    def __init__( self , appCfgObject):
        
        self.appConfig   = appCfgObject
        self.logFile     = logfile.Logfile(self.appConfig.resultFile_Path, 1)
        self.rResultFile = logfile.Logfile(self.appConfig.rformatFile_Path, 1)

    def writeFinalResult(self, markerList, globalPval, nmMarker, nmPermutations, startTime, endTime):

      
        # write basis data & parameter
        self.logFile.msgEoln("InputFile:\t%s" %self.appConfig.dataInputFile_Path )
        self.logFile.msgEoln("Number of hypotheses tests     :\t%i" %nmMarker)
        self.logFile.msgEoln("Number of perm. test statistics:\t%i" %nmPermutations)
        self.logFile.msgEoln("Start-Time:\t%s" %startTime)
        self.logFile.msgEoln("End  -Time:\t%s" %endTime)
        self.logFile.msgEoln("")
        self.logFile.msgEoln("TestCharacter        :\t%s" %self.appConfig.testCharacter)
        self.logFile.msgEoln("RawP - Formula       :\t%s" %self.appConfig.calcRawPFormula)
        self.logFile.msgEoln("Global pval - Formula:\t%s" %self.appConfig.calcGlobalPvalOption)
        self.logFile.msgEoln("")
        self.logFile.msgEoln("")

        # sort markers by ID + create headline for final reporting
        headline = ""
        dict = {}
        for m in markerList:
            dict[m['ln']] = m

        headline = "ID\tTval\tRawP\tAdjP\tGlobal"
        self.logFile.msgEoln(headline)

        if self.appConfig.resultRFormat:
            headline = "\t" + headline
            self.rResultFile.msgEoln(headline)
            
        
        # markers
        for i in range(len(markerList)):
            j = i + 1   # dict starts with key = 1
            m = dict[j]
            resMessage = str(m['id'])
            resMessage = resMessage + '\t' + str(m['tValOrg'])
            resMessage = resMessage + '\t' + str(m['pValRawOrg'])            
            resMessage = resMessage + '\t' + str(m['adjPval_enforcedMonotonicity'] )
            resMessage = resMessage + '\tNA' # thats the global            
            self.logFile.msgEoln(resMessage)

            if self.appConfig.resultRFormat:           
                rFormatMsg = str(j) + "\t" + resMessage
                self.rResultFile.msgEoln(rFormatMsg)

        # attach a last line with the global pvalue
        resMessage = "Global"
        for i in range(3):
            resMessage = resMessage + '\tNA'
        resMessage = resMessage + '\t' + str(globalPval)
        self.logFile.msgEoln(resMessage)
        
        rFormatMsg = str(j + 1) + "\t" + resMessage # !! j comes from upper section
        self.rResultFile.msgEoln(rFormatMsg)


        
    
