"""
Program  : SDMinP
Version  : 1.0
Date     : 16.03.2005
Purpose  : Adjustment of the multiple type I error (FWER)
Programer: Markus Obreiter
Contact  : m.obreiter@dkfz-heidelberg.de
Copyright (C) 2005 Deutsches Krebsforschungszentrum, DKFZ (German Cancer
Research Center), Division of Clinical
Epidemiology

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your option)
any later version. 
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

A copy of the GNU Public License is attached to the program documentation.
"""

import time, os, os.path, sys, string

class SplitFileControler:

    def __init__(self, appConfig):

        self.DEBUG = 0
        
        if appConfig:
            self.appConfig           = appConfig
            self.DATAINPUTPATH       = self.appConfig.dataInputFile_Path
            self.splitFile_Directory = self.appConfig.splitFile_Directory
            self.splitFile_PureName  = self.appConfig.splitFile_PureName
            self.splitFile_Extension = self.appConfig.splitFile_Extension
            self.splitFilesMaxLines  = self.appConfig.splitFilesMaxLines
            self.calculationID       = self.appConfig.calculationID
        else:
            self.DATAINPUTPATH       = "c:/test/test.txt"
            self.splitFile_Directory = "c:/test/split"
            self.splitFile_PureName  = "purname"
            self.splitFile_Extension = ".split"
            self.splitFilesMaxLines  = 4
            self.calculationID       = "mycalcId"
        
        if not os.path.isfile(self.DATAINPUTPATH):
            s = "File not found: %s" %self.DATAINPUTPATH
            sys.exit(s)            

        self.splitPathDict = {}

    def getSplitFilePath (self, number):
        splitFilePath  = "%s/%s_%s%i%s" %(self.splitFile_Directory, self.calculationID, self.splitFile_PureName, number, self.splitFile_Extension)
        return splitFilePath

    def removeFile (self, path):
        if os.path.isfile(path): os.remove(path)

    def writeSplitFiles(self):
        """splits the input data file in sub files.
           returns the number of lines of the original data file.
        """

        # empty the directory
        for file in os.listdir(self.splitFile_Directory):
            if os.path.splitext(file)[1] == self.splitFile_Extension:
                p = self.splitFile_Directory + os.sep + file
                os.remove(p)

        # check space on disk
        if self.appConfig.interactiveDialog:
            print
            print "Split files are going to be created"
            print "Max line number per split file: %i" %self.splitFilesMaxLines
            byteSize = float(os.path.getsize (self.DATAINPUTPATH))
            mbSize = byteSize / (1024 * 1024)
            
            if mbSize >= 1:
                print "At least %0.3f MB of free disc space are needed."  %mbSize
                if string.strip(raw_input("Are you sure you have enough space (type 'y')? ")) not in ("Y","y"):
                    sys.exit("Program terminated.")

        # create split files     
        inputFile = open (self.DATAINPUTPATH, "r")
        line = inputFile.readline()
        splitFileCount = 0
        splitFilePath  = self.getSplitFilePath(splitFileCount)
        self.removeFile (splitFilePath)
        splitFile = open(splitFilePath, "a")
        self.splitPathDict[splitFileCount] = splitFilePath
        lineCount      = 0
        totalLineCount = 0
        while line:
            lineCount      += 1
            totalLineCount += 1
            if self.DEBUG: print "Linecount %i" %lineCount
            if lineCount > self.splitFilesMaxLines:
                if self.DEBUG: print               
                if self.DEBUG: print "lineCount > maxLines"
                splitFile.close()
                splitFileCount += 1
                splitFilePath  = self.getSplitFilePath(splitFileCount)
                self.removeFile (splitFilePath)
                self.splitPathDict[splitFileCount] = splitFilePath
                splitFile = open(splitFilePath, "a")
                lineCount = 1
            if self.DEBUG: print "writing into --> %s" %os.path.basename(splitFilePath)
            splitFile.write(line)
            line = inputFile.readline()
        splitFile.close()
        inputFile.close()

        return totalLineCount
            
                       
    def display(self, message):
        print time.asctime() + " - " + message        

    def getLine(self, lineNumber):

        splitFileNumber  = (lineNumber-1) / self.splitFilesMaxLines
        if lineNumber <= self.splitFilesMaxLines:
            offsetLineNumber = lineNumber
        else:    
            offsetLineNumber = lineNumber % self.splitFilesMaxLines 

        if self.DEBUG: print lineNumber, splitFileNumber, offsetLineNumber
        
        if offsetLineNumber == 0: offsetLineNumber = self.splitFilesMaxLines
        splitFilePath    = self.splitPathDict[splitFileNumber]

        if self.DEBUG:
            print
            print "splitFileNumber %i" %splitFileNumber
            print "offsetLineNumber %i" %offsetLineNumber
            print "splitFilePath %s" %splitFilePath

        linecount = 0
        splitFile = open(splitFilePath, 'r')

        while linecount < offsetLineNumber:
            linecount += 1
            rLn = splitFile.readline()
        splitFile.close()
                                                
        return rLn           
            

        
        
